package com.hzya.frame.plugin.lets.util;

import com.hzya.frame.plugin.lets.dao.*;
import com.hzya.frame.plugin.lets.entity.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 档案类缓存，TOC转换
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.util
 * @Project：kangarooDataCenterV3
 * @name：BasicArchivesCacheUtil
 * @Date：2024/7/31 11:27
 * @Filename：BasicArchivesCacheUtil
 */
@Component
public class BasicArchivesCacheUtil {

    @Autowired
    private IBdCorpDao iBdCorpDao;

    @Autowired
    private IBdStordocDao iBdStordocDao;

    @Autowired
    private IBdCalbodyDao iBdCalbodyDao;

    @Autowired
    private IBdSalestruDao iBdSalestruDao;

    @Autowired
    private IBdDeptdocDao iBdDeptdocDao;

    @Autowired
    private IBdCumandocDao iBdCumandocDao;

    /**
     * 初始化公司档案-表头公司
     * 2024年7月31日 11:16:23
     */
    public static Map<String, BdCorpEntity> stringBdCorpEntityMap = new HashMap<>();

    public void initShop() throws Exception {
        BdCorpEntity bdCorpEntity = new BdCorpEntity();
        bdCorpEntity.setDr(0);
        bdCorpEntity.setDataSourceCode("sowow_sqlserver_test");
        List<BdCorpEntity> bdCorpEntityList = iBdCorpDao.query(bdCorpEntity);
        if (bdCorpEntityList != null && bdCorpEntityList.size() > 0) {
            for (int i = 0; i < bdCorpEntityList.size(); i++) {
                BdCorpEntity bdCorpEntity1 = bdCorpEntityList.get(i);
                stringBdCorpEntityMap.put(bdCorpEntity1.getUnitname(), bdCorpEntity1);
            }
        }
    }

    /**
     * 初始化发货仓库
     * 2024年7月31日 11:16:27
     * 202403011513：仓库根据仓库编码+库存组织编码去查询，之前是仓库编码+公司主键，和李佳妮确认好了
     */
    public static Map<String, BdStordocEntity> stringBdStordocEntityMap = new HashMap<>();

    public void initBdStordoc() throws Exception {
        BdStordocEntity bdStordocEntity = new BdStordocEntity();
        bdStordocEntity.setDr(0L);
        bdStordocEntity.setDataSourceCode("sowow_sqlserver_test");
        List<BdStordocEntity> bdStordocEntityList = iBdStordocDao.query(bdStordocEntity);
        if (bdStordocEntityList != null && bdStordocEntityList.size() > 0) {
            for (int i = 0; i < bdStordocEntityList.size(); i++) {
                BdStordocEntity bdStordocEntity1 = bdStordocEntityList.get(i);
                stringBdStordocEntityMap.put(bdStordocEntity1.getStorcode() + bdStordocEntity1.getPkCalbody(), bdStordocEntity1);
            }
        }
    }

    /**
     * 2024年7月31日 11:09:12 查询发货库存组织
     */
    public static Map<String, BdCalbodyEntity> stringBdCalbodyEntityMap = new HashMap<>();

    public void initBdCalbody() throws Exception {
        BdCalbodyEntity bdCalbodyEntity = new BdCalbodyEntity();
        bdCalbodyEntity.setDr(0);
        bdCalbodyEntity.setDataSourceCode("sowow_sqlserver_test");
        List<BdCalbodyEntity> bdCalbodyEntities = iBdCalbodyDao.query(bdCalbodyEntity);
        if (bdCalbodyEntities != null && bdCalbodyEntities.size() > 0) {
            for (int i = 0; i < bdCalbodyEntities.size(); i++) {
                BdCalbodyEntity bdCalbodyEntity1 = bdCalbodyEntities.get(i);
                stringBdCalbodyEntityMap.put(bdCalbodyEntity1.getPkCorp(), bdCalbodyEntity1);
            }
        }
    }

    /**
     * 初始化销售组织
     */
    public static Map<String, BdSalestruEntity> stringBdSalestruEntityMap = new HashMap<>();

    public void initBdSalestru() throws Exception {
        BdSalestruEntity bdSalestruEntity = new BdSalestruEntity();
        bdSalestruEntity.setDr(0);
        bdSalestruEntity.setDataSourceCode("sowow_sqlserver_test");
        List<BdSalestruEntity> bdSalestruEntityList = iBdSalestruDao.query(bdSalestruEntity);
        if (bdSalestruEntityList != null && bdSalestruEntityList.size() > 0) {
            for (int i = 0; i < bdSalestruEntityList.size(); i++) {
                BdSalestruEntity bdSalestruEntity1 = bdSalestruEntityList.get(i);
                stringBdSalestruEntityMap.put(bdSalestruEntity1.getBelongcorp(), bdSalestruEntity1);
            }
        }
    }

    /**
     * 初始化业务部门
     *
     * @author liuyang
     */
    public static Map<String, BdDeptdocEntity> stringBdDeptdocEntityMap = new HashMap<>();

    public void initDept() {
        BdDeptdocEntity bdDeptdocEntity = new BdDeptdocEntity();
        bdDeptdocEntity.setDataSourceCode("sowow_sqlserver_test");
        bdDeptdocEntity.setDr(0);
        bdDeptdocEntity.setDeptname("业务部门");
        List<BdDeptdocEntity> bdDeptdocEntityList = iBdDeptdocDao.query(bdDeptdocEntity);
        if (bdDeptdocEntityList != null && bdDeptdocEntityList.size() > 0) {
            for (int i = 0; i < bdDeptdocEntityList.size(); i++) {
                BdDeptdocEntity bdDeptdocEntity1 = bdDeptdocEntityList.get(i);
                stringBdDeptdocEntityMap.put(bdDeptdocEntity1.getPkCorp(), bdDeptdocEntity1);
            }
        }
    }

    /**
     * 初始化客商管理档案
     *
     * @author liuyang
     */
    public static Map<String, BdCumandocEntity> stringBdCumandocEntityMap = new HashMap<>();

    public void initBdCumandoc() {
        BdCumandocEntity bdCumandocEntity1 = new BdCumandocEntity();
        bdCumandocEntity1.setDataSourceCode("sowow_sqlserver_test");
        bdCumandocEntity1.setDr(0L);
//        bdCumandocEntity1.setCustflags(ProfilesActiveConstant.CUSTOMER);//客商:2客户
        List<BdCumandocEntity> bdCumandocEntityList = iBdCumandocDao.query(bdCumandocEntity1);
        if (bdCumandocEntityList != null && bdCumandocEntityList.size() > 0) {
            for (int i = 0; i < bdCumandocEntityList.size(); i++) {
                BdCumandocEntity bdCumandocEntity = bdCumandocEntityList.get(i);
                stringBdCumandocEntityMap.put(bdCumandocEntity.getPkCubasdoc() + bdCumandocEntity.getPkCorp(), bdCumandocEntity);
            }
        }
    }

    /**
     * 清理缓存
     *
     * @author liuyang
     */
    public void clearCache() throws Exception {
        stringBdCorpEntityMap.clear();
        stringBdStordocEntityMap.clear();
        stringBdCalbodyEntityMap.clear();
        stringBdSalestruEntityMap.clear();
        stringBdDeptdocEntityMap.clear();
        stringBdCumandocEntityMap.clear();
    }

    /**
     * 初始化缓存
     *
     * @author liuyang
     */
    public void initCache() throws Exception {
        clearCache();

        initShop();
        initBdStordoc();
        initBdCalbody();
        initBdSalestru();
        initDept();
        initBdCumandoc();
    }
}
